var createError = require('http-errors');
var express = require('express');
var path = require('path');
var cookieParser = require('cookie-parser');
var logger = require('morgan');
var cors = require('cors');

var indexRouter = require('./routes/index');
var usersRouter = require('./routes/users');
var apiRouter = require('./api/file');
var commonRouter = require('./routes/common');
var conf = require('./config/database');
const fileBucketRouter = require('./routes/files');

// Database Setup
const fs = require('fs');
const certFileBuf = fs.readFileSync('./rds-combined-ca-bundle.pem');

// Database Setup
const mongoose = require('mongoose');

mongoose.Promise = global.Promise;

// Connect MongoDB at default port 27017.
mongoose.connect(conf.database, {
  useNewUrlParser: true,
  useCreateIndex: true,
  sslCA: certFileBuf,
  useUnifiedTopology: true,
  useFindAndModify: false,
  poolSize: 100
}, (err) => {
  if (!err) {
    console.log('MongoDB Connection Succeeded.')
  } else {
    console.log('Error in DB connection: ' + err)
  }
});

var app = express();

// passport
const passport = require("passport");
app.use(passport.initialize());
app.use(passport.session());
require("./config/passport")(passport);
// passport end

// view engine setup
app.set('views', path.join(__dirname, 'views'));
app.set('view engine', 'ejs');

app.use(logger('dev'));

app.use(cors());

const file = require("./routes/file");
app.use("/api/file", file);

// app.use(express.json());
// app.use(express.urlencoded({ extended: false }));
app.use(express.json({ limit: '50mb' }));
app.use(express.urlencoded({ limit: '50mb', extended: false }));

app.use(cookieParser());

// Test route
const testRoute = require('./routes/test');
app.use('/test', testRoute);


// ==================================
const student = require("./routes/r_student");
app.use("/api/student", student);
// ==================================
// app.use('/', indexRouter);
app.use('/user', usersRouter);
app.use('/api', apiRouter);
app.use('/api/files', fileBucketRouter);
app.use('/common', commonRouter);

const mailRouter = require("./routes/mails");
app.use('/api/mail', mailRouter);

app.use(express.static(path.join(__dirname, 'public')));
app.use('/eams', express.static(path.join(__dirname, 'EAMS')));
// app.use('/eams/*', express.static(path.join(__dirname, 'eams')));
app.use('/', express.static(path.join(__dirname, 'eams-student')));
app.use('/*', express.static(path.join(__dirname, 'eams-student')));

// catch 404 and forward to error handler
app.use(function (req, res, next) {
  next(createError(404));
});

// error handler
app.use(function (err, req, res, next) {
  // set locals, only providing error in development
  res.locals.message = err.message;
  res.locals.error = req.app.get('env') === 'development' ? err : {};

  // render the error page
  res.status(err.status || 500);
  res.render('error');
});

module.exports = app;
