const mongoose = require('mongoose');
const config = require('../config/database');
const bcrypt = require('bcryptjs');

const UserSchema = mongoose.Schema({
  name: { type: String, required: true },
  username: { type: String, required: true },
  institute_code: { type: String, required: true },
  // gender: { type: String, required: true },
  gender: { type: String },
  mobile: { type: String },
  role: { type: String },
  email: { type: String },
  callerId: { type: String },
  wallet: { type: String, default: '0' },
  officeId: { type: Array, default: [] },
  cities: { type: Array, default: [] },
  offices: { type: Array, default: [] },
  role: { type: String },
  leads: { type: Array, default: [] },
  owner: { type: String, default: 'admin123' },
  password: { type: String, required: true },
  hashPassword: { type: String, required: true },
  lastRequest: {
    type: Date
  },
  lastUrl: {
    type: JSON,
    default: {
      server: {
        url: '',
        date: Date.now()
      },
      api: {
        url: '',
        type: '',
        date: Date.now()
      }
    }
  },
  created_at: {
    type: Date,
    default: Date.now()
  },
  created_by: {
    type: String,
  },
  updated_at: {
    type: Date,
  },
  updated_by: {
    type: String,
  },
}, {
  writeConcern: {
    w: 'majority',
    j: true,
    wtimeout: 20000
  }
});

const User = (module.exports = mongoose.model('User', UserSchema));

module.exports.addUser = (newUser, callback) => {
  newUser.save(callback);
};

module.exports.updateLastRoute = (id, url, method, routePath) => {
  User.findOneAndUpdate({ _id: id }, {
    $set: {
      'lastUrl': {
        server: {
          url: routePath,
          date: Date.now()
        },
        api: {
          url: url,
          type: method,
          date: Date.now()
        }
      }
    }
  }, (err, doc) => {
    // console.log(doc.lastUrl);
  });
};

module.exports.getUserByUsername = (username, callback) => {
  User.findOne({ username: username }, callback);
};

module.exports.getAllUsersCount = (callback) => {
  User.find(callback).countDocuments();
};

module.exports.getUserByIdForPassport = (id, callback) => {
  User.findOneAndUpdate({ _id: id }, { $set: { lastRequest: Date.now() } }, (err, doc) => {
    // console.log('log in tracked');
  });
  User.findById(id, callback);
};

module.exports.getUserById = (id, callback) => {
  User.findById(id, callback);
};

module.exports.getUsers = callback => {
  User.find(callback);
};

module.exports.getUsersSkipLimit = (skip, limit, callback) => {
  User.find(callback).skip(skip).limit(limit);
};

module.exports.getUsersByRole = (role, callback) => {
  User.find({ $and: [{ role: { $exists: true } }, { role: role }] }, callback);
};

module.exports.getUsersByOwner = (owner, callback) => {
  User.find({ owner: owner }, callback);
};

module.exports.getUsersByRoleAndOwner = (role, owner, callback) => {
  User.find({ $and: [{ owner: owner }, { role: role }] }, callback);
};

module.exports.getSuperAdmin = callback => {
  User.findOne({ role: 'super-admin' }, callback);
};

module.exports.getCounsellors = callback => {
  User.find({ role: 'counsellor' }, callback);
};

module.exports.getManagers = callback => {
  User.find({ role: 'manager' }, callback);
};

module.exports.getManagersOfAdmin = (username, callback) => {
  User.find({ owner: { admin: username } }, callback);
};

module.exports.getCounsellorsOfManager = (username, callback) => {
  User.find({ "owner.manager": username }, callback);
};

module.exports.getAdmins = callback => {
  User.find({ role: 'admin' }, callback);
};

module.exports.deleteUser = (username, callback) => {
  User.deleteOne({ username: username }, callback);
};

module.exports.comparePassword = (candidatePassword, hash, callback) => {
  bcrypt.compare(candidatePassword, hash, (err, isMatch) => {
    if (err) throw err;
    callback(null, isMatch);
  });
};
