var video = document.querySelector("#videoElement");
var canvas = document.querySelector('#canvas');
var photo = document.querySelector('#photo');
var examform = document.querySelector('#examform');
var width = 320;
var height = 0;
var streaming = false;
var imgData = null;

video.addEventListener('canplay', function (ev) {
    if (!streaming) {
        height = video.videoHeight / (video.videoWidth / width);

        video.setAttribute('width', width);
        video.setAttribute('height', height);
        canvas.setAttribute('width', width);
        canvas.setAttribute('height', height);
        streaming = true;
    }
}, false);

function start() {
    if (navigator.mediaDevices.getUserMedia) {
        navigator.mediaDevices.getUserMedia({ video: true })
            .then(function (stream) {
                video.srcObject = stream;
                takepicture();
            })
            .catch(function (err0r) {
                console.log("Something went wrong!");
            });
    }
}

function stop(e) {
    var stream = video.srcObject;
    var tracks = stream.getTracks();

    for (var i = 0; i < tracks.length; i++) {
        var track = tracks[i];
        track.stop();
    }

    video.srcObject = null;
    streaming = false;
}

function takepicture() {
    var context = canvas.getContext('2d');
    if (width && height) {
        canvas.width = width;
        canvas.height = height;
        context.drawImage(video, 0, 0, width, height);

        imgData = canvas.toDataURL('image/jpg');
        photo.setAttribute('src', imgData);
        setFace();
    
        if (streaming) {
            $.ajax({
                type: "POST",
                url: '/api/upload',
                data: { dataUrl: imgData },
                dataType: 'json',
                success: function (response) {
                    // console.log(response);
                    takepicture();
                },
                error : function(response) {
                    // console.log(response);
                    takepicture();
                }
            });
        } else {
            setTimeout(() => {
                takepicture();
            }, 1000);
        }
    } else {
        clearphoto();
    }
}

function clearphoto() {
    var context = canvas.getContext('2d');
    context.fillStyle = "#AAA";
    context.fillRect(0, 0, canvas.width, canvas.height);

    var data = canvas.toDataURL('image/png');
    photo.setAttribute('src', data);
    streaming = false;
}

function dataURItoBlob(dataURI) {
    // convert base64 to raw binary data held in a string
    // doesn't handle URLEncoded DataURIs - see SO answer #6850276 for code that does this
    var byteString = atob(dataURI.split(',')[1]);

    // separate out the mime component
    var mimeString = dataURI.split(',')[0].split(':')[1].split(';')[0]

    // write the bytes of the string to an ArrayBuffer
    var ab = new ArrayBuffer(byteString.length);

    // create a view into the buffer
    var ia = new Uint8Array(ab);

    // set the bytes of the buffer to the correct values
    for (var i = 0; i < byteString.length; i++) {
        ia[i] = byteString.charCodeAt(i);
    }

    // write the ArrayBuffer to a blob, and you're done
    var blob = new Blob([ab], { type: mimeString });
    return blob;

}

function setFace() {

    var tracker = new tracking.ObjectTracker(['face']);
    tracker.setStepSize(1.7);

    tracking.track('#photo', tracker);

    tracker.on('track', function (event) {
        if (event.data.length) {
            showAlert(`${event.data.length} Faces Found`);
            examform.classList.remove('d-none');
        } else {
            showAlert(`Face Not Found`);
            examform.classList.add('d-none');
        }
        event.data.forEach(function (rect) {
            window.plot(rect.x, rect.y, rect.width, rect.height);
        });
    });

    window.plot = function (x, y, w, h) {
        var rects = document.querySelectorAll('.rect');
        for (const r of rects) {
            var o = document.querySelector('.output');
            o.removeChild(r);
        }
        var rect = document.createElement('div');
        document.querySelector('.output').appendChild(rect);
        rect.classList.add('rect');
        rect.style.width = w + 'px';
        rect.style.height = h + 'px';
        rect.style.left = (photo.offsetLeft + x) + 'px';
        rect.style.top = (photo.offsetTop + y) + 'px';
    };

}

function showAlert(msg) {
    document.querySelector("#alert-box").innerText = msg;
}

function checkPageFocus() {
    if (!document.hasFocus()) {
        alert("Your Exam is cancelled");
    }
}

// setInterval(checkPageFocus, 1000);

start();