const Models = require('./../models/common');
const router = require('express').Router();
const helper = require('./../Helpers/handler');
const e = require('express');
const config = require('./../config/database');
const jwt = require('jsonwebtoken');
const passport = require('passport');
var path = require("path");
var xlsx = require("node-xlsx");
var moment = require('moment');

router.post('/:term',
    // passport.authenticate('jwt', { session: false }),
    (req, res) => {
        // console.log(req.user);
        const label = req.params.term;
        const Table = getModel(label)[0];
        const limit = req.body.limit ? req.body.limit : 10;
        const page = req.body.page ? req.body.page : 1;
        const skip = (page * limit) - limit;
        if (Table) {
            Table.TableModel.countDocuments((err, docsLength) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    Table.TableModel.find((err, docs) => {

                        if (err) {
                            helper.errHandler(res, err);
                        } else {
                            res.json({
                                success: true,
                                msg: 'Data Fetched',
                                data: {
                                    docs,
                                    docsLength
                                }
                            });
                        }
                    }).skip(skip).limit(limit).maxTimeMS(20000);
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/query',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        // console.log(req.user);
        const label = req.params.term;

        const query = req.body.query;
        if (req.user.username != 'eams-admin' && req.user.role != 'exam-controller' && req.user.role != 'teacher') {
            if (label != 'batch' && label != 'course' && label != 'subject') {
                if (label == 'timetable') {
                    query.institute_code = req.user.institute_code;
                } else {
                    // query.created_by = req.user._id;
                    query.institute_code = req.user.institute_code;
                }
            }
        }

        const Table = getModel(label)[0];
        const limit = req.body.limit ? req.body.limit : 10;
        const page = req.body.page ? req.body.page : 1;
        const skip = (page * limit) - limit;
        // for (const key of Object.keys(query)) {
        //     if (key != "_id") {
        //         query[key] = new RegExp(query[key], 'gi');
        //     }
        // }
        if (Table) {
            if (label == 'questionbank') {
                Table.TableModel.countDocuments(query, (err, docsLength) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        Table.TableModel.find(query, (err, docs) => {

                            if (err) {
                                helper.errHandler(res, err);
                            } else {
                                res.json({
                                    success: true,
                                    msg: 'Data Fetched',
                                    data: {
                                        docs,
                                        docsLength
                                    }
                                });
                            }
                        }).sort({ level: -1 }).skip(skip).limit(limit).maxTimeMS(20000);
                    }
                });
            } else {
                Table.TableModel.countDocuments(query, (err, docsLength) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        Table.TableModel.find(query, (err, docs) => {
                            if (err) {
                                helper.errHandler(res, err);
                            } else {
                                res.json({
                                    success: true,
                                    msg: 'Data Fetched',
                                    data: {
                                        docs,
                                        docsLength
                                    }
                                });
                            }
                        }).sort({ _id: -1 }).skip(skip).limit(limit).maxTimeMS(20000);
                    }
                });
            }
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/search',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;

        const query = req.body.query;
        if (req.user.username != 'eams-admin') {
            if (label != 'batch' && label != 'course' && label != 'subject') {
                if (label == 'timetable') {
                    query.institute_code = req.user.institute_code;
                } else {
                    // query.created_by = req.user._id;
                    query.institute_code = req.user.institute_code;
                }
            }
        }

        const Table = getModel(label)[0];
        const limit = req.body.limit ? req.body.limit : 10;
        const page = req.body.page ? req.body.page : 1;
        const skip = (page * limit) - limit;
        for (const key of Object.keys(query)) {
            if (key != "_id") {
                query[key] = new RegExp(query[key], 'gi');
            }
        }
        if (Table) {
            if (label == 'questionbank') {
                Table.TableModel.countDocuments(query, (err, docsLength) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        Table.TableModel.find(query, (err, docs) => {
                            if (err) {
                                helper.errHandler(res, err);
                            } else {
                                res.json({
                                    success: true,
                                    msg: 'Data Fetched',
                                    data: {
                                        docs,
                                        docsLength
                                    }
                                });
                            }
                        }).sort({ level: -1 }).skip(skip).limit(limit).maxTimeMS(20000);
                    }
                });
            } else {
                Table.TableModel.countDocuments(query, (err, docsLength) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        Table.TableModel.find(query, (err, docs) => {
                            if (err) {
                                helper.errHandler(res, err);
                            } else {
                                res.json({
                                    success: true,
                                    msg: 'Data Fetched',
                                    data: {
                                        docs,
                                        docsLength
                                    }
                                });
                            }
                        }).sort({ _id: -1 }).skip(skip).limit(limit).maxTimeMS(20000);
                    }
                });
            }
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/create',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const doc = req.body.doc;
        doc.created_at = Date.now();
        doc.created_by = req.user._id;
        if (!doc.institute_code) {
            doc.institute_code = req.user.institute_code;
        }
        const Table = getModel(label)[0];

        if (!doc) {
            return helper.errHandler(res, { message: "Can't insert blank entry" });
        }

        if (Table) {
            if (label == 'student') {
                // doc.ip = req.connection.remoteAddress;
                // doc.ip = req.ip;
                doc.ip = req.headers['x-real-ip'];
                if (!doc.branch && !doc.subjects) {
                    return res.json({ success: false, msg: "INVALID REQUEST" });
                }
            }
            Table.TableModel.create(doc, { wtimeout: 20000 }, (err, doc) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    res.json({
                        success: true,
                        msg: 'Data Created',
                        data: {
                            doc
                        }
                    });
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/bulkcreate',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const docs = req.body.docs;
        const Table = getModel(label)[0];

        for (const doc of docs) {
            doc.created_at = Date.now();
            doc.created_by = req.user._id;
            doc.institute_code = req.user.institute_code;
        }

        if (!docs) {
            return helper.errHandler(res, { message: "Can't insert blank entry" });
        }
        if (!docs.length) {
            return helper.errHandler(res, { message: "Can't insert blank entry" });
        }

        if (Table) {
            Table.TableModel.insertMany(docs, (err, docs) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    res.json({
                        success: true,
                        msg: 'Data Created',
                        data: {
                            docsCount: docs.length
                        }
                    });
                }
            })
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/update',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const query = req.body.query;
        const doc = req.body.doc;
        doc.updated_by = req.user._id;
        const Table = getModel(label)[0];
        if (Table) {
            Table.TableModel.updateOne(query, { $set: doc }, (err, doc) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    res.json({
                        success: true,
                        msg: 'Data Updated',
                        data: {
                            doc
                        }
                    });
                }
            })
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/updatebyid',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const id = req.body.id;
        const doc = req.body.doc;
        doc.updated_by = req.user._id;

        for (const key of Object.keys(doc)) {
            if (key == "registerationIp") {
                doc[key] = req.headers['x-real-ip'] ? req.headers['x-real-ip'] : '-';
            }
        }
        const Table = getModel(label)[0];
        if (Table) {
            Table.TableModel.findByIdAndUpdate(id, { $set: doc }, (err, doc) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    res.json({
                        success: true,
                        msg: 'Data Updated',
                        data: {
                            doc
                        }
                    });
                }
            })
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/delete',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const query = req.body.query;
        const Table = getModel(label)[0];
        if (Table) {
            Table.TableModel.find(query, (err, docs) => {
                if (err) { helper.errHandler(res, err) }
                else {
                    Table.OldTableModel.insertMany(docs, (err, docs) => {
                        if (err) { helper.errHandler(res, err) }
                        else {
                            Table.TableModel.deleteMany(query, (err, doc) => {
                                if (err) {
                                    helper.errHandler(res, err);
                                } else {
                                    res.json({
                                        success: true,
                                        msg: 'Data Deleted',
                                        data: {
                                            doc
                                        }
                                    });
                                }
                            });
                        }
                    });
                }
            }).maxTimeMS(20000)
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/deletebyid',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const id = req.body.id;
        const Table = getModel(label)[0];
        if (Table) {
            Table.TableModel.findById(id, (err, doc) => {
                if (err) { helper.errHandler(res, err) }
                else {
                    Table.OldTableModel.insertMany(doc, (err, doc) => {
                        if (err) { helper.errHandler(res, err) }
                        else {
                            Table.TableModel.findByIdAndDelete(id, (err, doc) => {
                                if (err) {
                                    helper.errHandler(res, err);
                                } else {
                                    res.json({
                                        success: true,
                                        msg: 'Data Deleted',
                                        data: {
                                            doc
                                        }
                                    });
                                }
                            });
                        }
                    });
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.get('/:term/updateBranchAndBatch',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;
        const Table = getModel(label)[0];
        if (Table && Table.TableName == 'Student') {
            Table.TableModel.find((err, docs) => {
                if (err) { helper.errHandler(res, err) }
                else {
                    res.json({ success: true, msg: "Updating....." });
                    for (const doc of docs) {
                        doc.branch_name = doc.enrollment_no.substr(4, 2);
                        doc.batch_code = doc.course + '-' + doc.enrollment_no.substr(4, 2);
                        Table.TableModel.findOneAndUpdate({ _id: doc._id }, { $set: doc }, (err, doc) => {
                            if (err) {
                                console.log(err);
                            } else {
                                console.log("updated");
                            }
                        });
                    }
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/paper/:papercode',
    passport.authenticate('jwt', { session: false }),
    async (req, res) => {
        // console.log(req.user);
        const paperCode = req.params.papercode;
        if (typeof paperCode != "string") {
            return res.json({
                success: false,
                msg: "PaperCode Needed to be in correct format",
                data: {
                    status: false
                }
            });
        }

        // const eno = req.body.enrollment_no;
        const eno = req.user.enrollment_no;
        const marks = req.params.marks;
        let questions = [];
        let filteredQuestions = [];
        let levels = {};
        const tpTable = getModel('testpaper')[0];
        const rsTable = getModel('result')[0];
        const stdTable = getModel('student')[0];

        const student = await stdTable.TableModel.findOne({ enrollment_no: eno }).maxTimeMS(20000);
        if (!student) {
            return res.json({ success: false, msg: "student data not found", data: { status: false } });
        }
        if (rsTable) {
            rsTable.TableModel.findOne({ paper_id: paperCode, "student.enrollment_no": eno }, (err, docs) => {
                if (err) {
                    return helper.errHandler(res, err);
                } else {
                    if (docs) {
                        getModel("questionmap")[0].TableModel.findOne({ paper_code: paperCode, eno }, async (err, queToSend) => {
                            if (err) {
                                return helper.errHandler(res, err);
                            } else {
                                if (queToSend) {
                                    const questions_answered = await getModel("questionanswer")[0].TableModel.find({ paper_code: paperCode, eno }).maxTimeMS(20000);
                                    const questions_seen = await getModel("questionseen")[0].TableModel.find({ paper_code: paperCode, eno }).maxTimeMS(20000);
                                    // console.log(questions_answered);
                                    // console.log(questions_seen);
                                    return res.json({
                                        success: true,
                                        msg: 'Data Fetched',
                                        data: {
                                            result: docs,
                                            queToSend: queToSend.ques,
                                            questions_answered,
                                            questions_seen: questions_seen.map(o => o.qid)
                                        }
                                    })
                                } else {
                                    return res.json({
                                        success: false,
                                        msg: "Network Error.",
                                        data: {
                                            status: false
                                        }
                                    });
                                }
                            }
                        }).maxTimeMS(20000);
                    } else {
                        if (tpTable) {
                            // console.log(student.institute_code);
                            tpTable.TableModel.findOne({ test_paper_name: paperCode, institute_code: student.institute_code }, (err, docs) => {
                                if (err) {
                                    helper.errHandler(res, err);
                                } else {
                                    // console.log(docs);
                                    if (!docs) {
                                        return res.json({
                                            success: false,
                                            msg: "Test Paper Awaited",
                                            data: {
                                                status: false
                                            }
                                        });
                                    }
                                    const paper = docs;
                                    questions = paper.question_codes;
                                    let levelCheck = []
                                    for (let index = 1; index <= 10; index++) {
                                        const q = 'question_of_' + index;
                                        if (paper[q] > 0) {
                                            levels[index] = paper[q];
                                            levelCheck.push(index);
                                        }
                                    }
                                    // const questionQuery = {
                                    //     _id: { $in: questions },
                                    //     level: { $in: levelCheck, },
                                    // };
                                    const qTable = getModel('questionbank')[0];
                                    if (qTable) {
                                        // console.log('Question ID : ', ques);
                                        qTable.TableModel.find({ class_name: paperCode, institute_code: student.institute_code, level: { $in: levelCheck } }, {
                                            _id: 1,
                                            question: 1,
                                            options: 1,
                                            question_hindi: 1,
                                            options_hindi: 1,
                                            is_hindi: 1,
                                            is_image: 1,
                                            image_path: 1,
                                            class_name: 1,
                                            level: 1,
                                            institute_code: 1
                                        }, async (err, ques) => {
                                            if (err) {
                                                helper.errHandler(res, err);
                                            } else {
                                                // console.log(ques);
                                                if (!ques.length) {
                                                    return res.json({
                                                        success: false,
                                                        msg: "no questions can be alloted to you",
                                                        data: {
                                                            status: false
                                                        }
                                                    });
                                                }

                                                let quechecks = {};

                                                ques = shuffle(ques);

                                                for (const que of ques) {
                                                    for (const lev of levelCheck) {
                                                        if (que.level == lev) {
                                                            que.correct_answer = "";
                                                            que.options = que.options.map(opt => {
                                                                // * Null Check
                                                                return opt ? (opt).toString() : "false";
                                                            });
                                                            quechecks[lev] = quechecks[lev] ? quechecks[lev] : [];
                                                            quechecks[lev] = [...quechecks[lev], que];
                                                            if (quechecks[lev].length >= levels[lev]) {
                                                                levelCheck = levelCheck.filter(o => o != lev);
                                                            }
                                                            break;
                                                        }
                                                    }
                                                }

                                                // for (const lev of levelCheck) {
                                                //     quechecks[lev] = ques.filter(o => o.level == lev);
                                                //     quechecks[lev] = quechecks[lev].slice(0, levels[lev]);
                                                //     quechecks[lev] = shuffle(quechecks[lev]);
                                                // }

                                                // ques = ques.map(o => {
                                                //     o.correct_answer = "";
                                                //     /**
                                                //      * * Parse Every Option To String
                                                //      * Number(23) -> String("23")
                                                //      * Boolean(false) -> String("false")
                                                //      *  */
                                                //     o.options = o.options.map(opt => {
                                                //         // * Null Check
                                                //         return opt ? (opt).toString() : "";
                                                //     });
                                                //     return o;
                                                // });

                                                let queToSend = [];
                                                for (const key of Object.keys(quechecks)) {
                                                    queToSend = [...queToSend, ...quechecks[key]];
                                                }
                                                queToSend = shuffle(queToSend);

                                                try {
                                                    const questionMapped = await getModel("questionmap")[0].TableModel.create({
                                                        eno: eno,
                                                        paper_code: paperCode,
                                                        ques: queToSend,
                                                        institute_code: student.institute_code,
                                                        created_at: Date.now()
                                                    });
                                                    if (questionMapped) {
                                                        rsTable.TableModel.create({
                                                            exam_id: paper._id,
                                                            paper_id: paperCode,
                                                            student_id: student._id,
                                                            // student_photos: [],
                                                            // answers: [],
                                                            // questions_mapped: queToSend,
                                                            questions_total: queToSend.map((o) => o._id),
                                                            // questions_seen: [queToSend[0]._id],
                                                            // questions_unseen: this.shareds.questions.map(o => o._id),
                                                            // questions_answered: [],
                                                            warnings_left: 0,
                                                            last_warning: "",
                                                            student: {
                                                                _id: student._id,
                                                                name: student.name,
                                                                enrollment_no: student.enrollment_no
                                                            },
                                                            login_attempts: 0,
                                                            submitted: false,
                                                            institute_code: student.institute_code,
                                                            created_at: Date.now()
                                                        }, (err, result) => {
                                                            if (err) {
                                                                return helper.errHandler(res, err);
                                                            } else {
                                                                if (ques) {
                                                                    res.json({
                                                                        success: true,
                                                                        msg: 'Data Fetched',
                                                                        data: {
                                                                            result,
                                                                            queToSend,
                                                                            questions_answered: [],
                                                                            questions_seen: [queToSend[0]._id]
                                                                        }
                                                                    });
                                                                } else {
                                                                    helper.errHandler(res, { message: 'Questions not found' });
                                                                }
                                                            }
                                                        });
                                                    } else {
                                                        return res.json({
                                                            success: false,
                                                            msg: "Can't allot questions to you",
                                                            data: {
                                                                status: false
                                                            }
                                                        });
                                                    }
                                                } catch (error) {
                                                    return res.json({
                                                        success: false,
                                                        msg: "Network Error",
                                                        data: {}
                                                    });
                                                }
                                            }
                                        }).maxTimeMS(20000);
                                    } else {
                                        helper.errHandler(res, { message: 'Table qp Not Found' });
                                    }
                                }
                            }).maxTimeMS(20000);
                        } else {
                            helper.errHandler(res, { message: 'Table tp Not Found' });
                        }
                    }
                }
            }).maxTimeMS(20000);
        }
        else {
            return helper.errHandler(res, { message: 'Table rs Not Found' });
        }
    });

// Student Routes
// 
router.post('/:term/updateResult/:paperCode',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;

        const query = req.body.query;
        const paperCode = req.params.paperCode

        query.enrollment_no = req.user.enrollment_no;
        query.institute_code = req.user.institute_code;

        const Table = getModel('result')[0];

        if (Table) {
            Table.TableModel.updateOne({ "student.enrollment_no": query.enrollment_no, paper_id: paperCode }, {
                $set: {
                    warnings_left: query.warnings_left,
                    submitted: query.submitted ? true : false
                }
            }, (err, docs) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    res.json({
                        success: true,
                        msg: 'Data Fetched',
                        data: {}
                    });
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/updateSeen/:paperCode',
    passport.authenticate('jwt', { session: false }),
    async (req, res) => {
        const label = req.params.term;

        const query = req.body.query;
        const paperCode = req.params.paperCode;

        query.enrollment_no = req.user.enrollment_no;
        query.institute_code = req.user.institute_code;

        const Table = getModel('questionseen')[0];

        const student = await getModel('student')[0].TableModel.findOne({ enrollment_no: query.enrollment_no }, { institute_code: 1 }).maxTimeMS(20000);

        if (!student) {
            return res.json({
                success: false,
                msg: "Student Data not found",
                data: {}
            });
        }

        if (Table) {

            const QuestionSeenObject = await Table.TableModel.findOne({ eno: query.enrollment_no, qid: query.qid, paper_code: paperCode }, { institute_code: 1 }).maxTimeMS(20000);
            if (!QuestionSeenObject) {
                Table.TableModel.create({
                    paper_code: paperCode,
                    eno: query.enrollment_no,
                    paper_code: paperCode,
                    qid: query.qid,
                    institute_code: query.institute_code,
                    created_at: Date.now()
                }, (err, docs) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        return res.json({
                            success: true,
                            msg: 'Question Seen',
                            data: {}
                        });
                    }
                });
            } else {
                return res.json({
                    success: true,
                    msg: 'Question Seen',
                    data: {}
                });
            }
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/updateAnswered/:paperCode',
    passport.authenticate('jwt', { session: false }),
    async (req, res) => {
        const label = req.params.term;

        const query = req.body.query;
        const paperCode = req.params.paperCode;

        query.enrollment_no = req.user.enrollment_no;
        query.institute_code = req.user.institute_code;

        const Table = getModel('questionanswer')[0];

        const student = await getModel('student')[0].TableModel.findOne({ enrollment_no: query.enrollment_no }, { institute_code: 1 }).maxTimeMS(20000);

        if (!student) {
            return res.json({
                success: false,
                msg: "Student Data not found",
                data: {}
            });
        }

        if (Table) {
            const QuestionAnsweredObject = await Table.TableModel.findOne({ eno: query.enrollment_no, qid: query.qid, paper_code: paperCode }, { institute_code: 1 }).maxTimeMS(20000);

            if (!QuestionAnsweredObject) {
                Table.TableModel.create({
                    paper_code: paperCode,
                    eno: query.enrollment_no,
                    paper_code: paperCode,
                    qid: query.qid,
                    que: query.que,
                    ans: query.ans,
                    institute_code: query.institute_code,
                    created_at: Date.now()
                }, (err, docs) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        res.json({
                            success: true,
                            msg: 'Question Answered',
                            data: {}
                        });
                    }
                });
            } else {
                Table.TableModel.updateOne({
                    eno: query.enrollment_no, qid: query.qid, paper_code: paperCode
                }, {
                    $set: { ans: query.ans }
                }, (err, docs) => {
                    if (err) {
                        helper.errHandler(res, err);
                    } else {
                        res.json({
                            success: true,
                            msg: 'Question Answered',
                            data: {}
                        });
                    }
                });
            }
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/urlcheck',
    (req, res) => {

        const query = req.body.query;

        if ((typeof (query)) == "object") {
            if (!query.enrollment_no || !query.otp || (typeof (query.enrollment_no)) != 'string' || (typeof (query.otp)) != 'string') {
                return res.json({
                    success: false,
                    msg: "Enrollment Number OR OTP is missing",
                    data: {
                        status: false,
                        docsLength: 0
                    }
                });
            }
        } else {
            return res.json({
                success: false,
                msg: "Enrollment Number OR OTP with query is missing",
                data: {
                    status: false,
                    docsLength: 0
                }
            });
        }

        const Table = getModel('student')[0];

        if (Table) {
            Table.TableModel.countDocuments({ enrollment_no: query.enrollment_no, otp: query.otp }, (err, docsLength) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    Table.TableModel.find(query, (err, docs) => {
                        if (err) {
                            helper.errHandler(res, err);
                        } else {
                            return res.json({
                                success: true,
                                msg: 'Data Fetched',
                                data: {
                                    status: docsLength ? true : false,
                                    docsLength
                                }
                            });
                        }
                    }).maxTimeMS(20000);
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/getresult',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;

        const query = req.body.query;

        query.enrollment_no = req.user.enrollment_no;
        query.institute_code = req.user.institute_code;

        if ((typeof (query)) == "object") {
            if (!query.enrollment_no || (typeof (query.enrollment_no)) != 'string') {
                return res.json({
                    success: false,
                    msg: "Enrollment Number is missing",
                    data: {
                        status: false,
                        docsLength: 0
                    }
                });
            }
        } else {
            return res.json({
                success: false,
                msg: "Enrollment Number with query is missing",
                data: {
                    status: false,
                    docsLength: 0
                }
            });
        }

        const Table = getModel('result')[0];

        if (Table) {
            Table.TableModel.countDocuments({ "student.enrollment_no": query.enrollment_no }, (err, docsLength) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    console.log(query);
                    Table.TableModel.find({ "student.enrollment_no": query.enrollment_no, "paper_id": query.paper_id, "institute_code": query.institute_code, }, (err, docs) => {
                        if (err) {
                            helper.errHandler(res, err);
                        } else {
                            res.json({
                                success: true,
                                msg: 'Data Fetched',
                                data: {
                                    docs: docs,
                                    docsLength
                                }
                            });
                        }
                    }).maxTimeMS(20000);
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/confirmprofile',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;

        const query = req.body.query;

        query.enrollment_no = req.user.enrollment_no;
        query.institute_code = req.user.institute_code;

        if ((typeof (query)) == "object") {
            if (!((Object.keys(query)).length)) {
                return res.json({
                    success: false,
                    msg: "Parameter is missing",
                    data: {
                        status: false,
                        docsLength: 0
                    }
                });
            }
        } else {
            return res.json({
                success: false,
                msg: "Parameter with query is missing",
                data: {
                    status: false,
                    docsLength: 0
                }
            });
        }

        const Table = getModel('student')[0];

        if (Table) {
            Table.TableModel.countDocuments(query, (err, docsLength) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    Table.TableModel.find(query, (err, docs) => {
                        if (err) {
                            helper.errHandler(res, err);
                        } else {
                            if (docs.length) {
                                Table.TableModel.updateOne(query, { $set: { ip: req.headers['x-real-ip'] } }, (err, doc) => { });
                                const student = {
                                    name: docs[0].name,
                                    ip: req.headers['x-real-ip'],
                                    institute_name: docs[0].institute_name
                                }
                                return res.json({
                                    success: true,
                                    msg: 'Data Fetched',
                                    data: {
                                        status: true,
                                        docs: [student],
                                        docsLength
                                    }
                                });
                            } else {
                                return res.json({
                                    success: false,
                                    msg: 'Student Data Mismatch',
                                    data: {
                                        status: false,
                                        docsLength
                                    }
                                });
                            }
                        }
                    }).maxTimeMS(20000);
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/checkdob',
    (req, res) => {
        const label = req.params.term;

        const query = req.body.query;

        if ((typeof (query)) == "object") {
            if (!query.enrollment_no || !query.dob || !query.father_name || !query.otp || (typeof (query.enrollment_no)) != 'string' || (typeof (query.dob)) != 'string' || (typeof (query.father_name)) != 'string' || (typeof (query.otp)) != 'string') {
                return res.json({
                    success: false,
                    msg: "Enrollment Number OR DOB OR Father Name OR otp is missing",
                    data: {
                        status: false,
                        docsLength: 0
                    }
                });
            }
        } else {
            return res.json({
                success: false,
                msg: "Enrollment Number OR DOB OR Father Name OR otp with query is missing",
                data: {
                    status: false,
                    docsLength: 0
                }
            });
        }

        for (const key of Object.keys(query)) {
            if (key == "father_name") {
                query[key] = new RegExp(query[key], 'gi');
            }
        }

        const Table = getModel('student')[0];

        if (Table) {
            Table.TableModel.findOne({
                enrollment_no: query.enrollment_no,
                dob: query.dob,
                father_name: query.father_name,
                otp: query.otp
            }, (err, doc) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    if (!doc) {
                        return res.json({
                            success: true,
                            msg: 'Data Fetched',
                            data: {
                                status: false
                            }
                        });
                    } else {

                        var t_user = {
                            id: doc._id,
                            enrollment_no: doc.enrollment_no,
                            dob: doc.dob
                        };
                        const token = jwt.sign(t_user, config.secret, {
                            expiresIn: 10800
                        });

                        return res.json({
                            success: true,
                            msg: 'Data Fetched',
                            data: {
                                status: true,
                                token
                            }
                        });
                    }
                }
            }).maxTimeMS(20000);
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.post('/:term/checkexamconfig',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const label = req.params.term;

        const query = req.body.query;

        if ((typeof (query)) == "object") {
            if (!((Object.keys(query)).length)) {
                return res.json({
                    success: false,
                    msg: "Parameter is missing",
                    data: {
                        status: false,
                        docsLength: 0
                    }
                });
            }
        } else {
            return res.json({
                success: false,
                msg: "Parameter with query is missing",
                data: {
                    status: false,
                    docsLength: 0
                }
            });
        }

        const Table = getModel('student')[0];

        let today_date = new Date();

        query.enrollment_no = req.user.enrollment_no;

        // *TODO: Uncomment two lines for timezone
        today_date.setHours(today_date.getHours() + 5);
        today_date.setMinutes(today_date.getMinutes() + 30);

        if (Table) {
            Table.TableModel.countDocuments(query, (err, docsLength) => {
                if (err) {
                    helper.errHandler(res, err);
                } else {
                    Table.TableModel.find(query, (err, docs) => {
                        if (err) {
                            helper.errHandler(res, err);
                        } else {
                            if (docs && docs.length && docs[0].subjects && docs[0].subjects.length && (docs[0].subjects[0]).split(',')) {
                                let student = docs[0];
                                let subjects = (docs[0].subjects[0]).split(',');

                                const TimeTableTable = getModel('timetable')[0];

                                TimeTableTable.TableModel.find({ paper_code: { $in: subjects }, institute_code: student.institute_code }, (err, docs) => {
                                    if (err) {
                                        helper.errHandler(res, err);
                                    } else {
                                        // console.log(subjects);
                                        // console.log(student.institute_code);
                                        if (docs.length) {
                                            let timetables = docs;
                                            // console.log(timetables.map(o => o.paper_code));
                                            // console.log(timetables.map(o => o.subject));
                                            let currenttimetable = [];
                                            // Checking Dates
                                            for (const tt of timetables) {
                                                const temp_date = tt.exam_date.split("/");
                                                const exam_date = new Date(`${temp_date[2]}-${temp_date[1]}-${temp_date[0]}`);
                                                const diffTime = exam_date - today_date;
                                                const diffDays = Math.ceil(diffTime / (1000 * 60 * 60 * 24));
                                                // console.log((diffTime / (1000 * 60 * 60)) + " hour");
                                                // console.log(diffDays + " days");
                                                // *TODO: make difference to 0
                                                if (diffDays == 0) {
                                                    currenttimetable.push(tt);
                                                }
                                            }
                                            // If No Exam Today
                                            if (!currenttimetable.length) {
                                                return res.json({
                                                    success: true,
                                                    msg: 'No Exam Scheduled today!',
                                                    data: {
                                                        status: false,
                                                        docsLength
                                                    }
                                                });
                                            }
                                            // Checking Time
                                            let todaytimetable = {};
                                            for (const ctt of currenttimetable) {
                                                // console.log(ctt.exam_start_time);
                                                // console.log(ctt.exam_end_time);
                                                todaytimetable = ctt;
                                                let start_time = ctt.exam_start_time.split(":");
                                                start_time[0] = parseInt(start_time[0]);
                                                start_time[1] = parseInt(start_time[1]);
                                                if (start_time[1] == 0) {
                                                    start_time[0] = start_time[0] - 1;
                                                    start_time[1] = 45;
                                                } else if (start_time[1] >= 15) {
                                                    start_time[0] = start_time[0];
                                                    start_time[1] = start_time[1] - 15;
                                                } else {
                                                    return res.json({
                                                        success: true,
                                                        msg: 'Invalid Exam Time In Time Table!',
                                                        data: {
                                                            status: false,
                                                            docsLength
                                                        }
                                                    });
                                                }
                                                let end_time = ctt.exam_end_time.split(":");
                                                end_time[0] = parseInt(end_time[0]);
                                                end_time[1] = parseInt(end_time[1]);
                                                // console.log(start_time);
                                                // console.log(end_time);
                                                // console.log(today_date.getHours());
                                                // console.log(today_date.getMinutes());
                                                let isExamPossible = true;
                                                let reason = '';
                                                let secs = 20;
                                                let gap = 90 * 60;
                                                if (today_date.getHours() < start_time[0]) {
                                                    isExamPossible = false;
                                                    reason = `You are ahead of reporting time, Reporting time is ${start_time[0]}:${start_time[1]}`;
                                                } else if (today_date.getHours() == start_time[0]) {
                                                    if (today_date.getMinutes() < start_time[1]) {
                                                        isExamPossible = false;
                                                        reason = `You are ahead of reporting time, Reporting time is ${start_time[0]}:${start_time[1]}`;
                                                    } else {
                                                        const minDiff = today_date.getMinutes() - start_time[1];
                                                        if ((15 - minDiff) > 0) {
                                                            secs = (15 - minDiff) * 60;
                                                        }
                                                        isExamPossible = true;
                                                    }
                                                } else if (today_date.getHours() < end_time[0]) {
                                                    isExamPossible = true;
                                                } else if (today_date.getHours() == end_time[0]) {
                                                    if (today_date.getMinutes() >= end_time[1]) {
                                                        isExamPossible = false;
                                                        reason = `Your exam is over at ${end_time[0]}:${end_time[1]}`;
                                                    } else {
                                                        isExamPossible = true;
                                                    }
                                                } else if (today_date.getHours() > end_time[0]) {
                                                    isExamPossible = false;
                                                    reason = `Your exam is over at ${end_time[0]}:${end_time[1]}`;
                                                }
                                                const h_gap = end_time[0] - today_date.getHours();
                                                const m_gap = today_date.getMinutes() - end_time[1];
                                                if (h_gap > 0) {
                                                    gap = ((h_gap * 60) - m_gap) * 60;
                                                } else {
                                                    gap = (m_gap) * 60;
                                                }
                                                if (isExamPossible) {
                                                    return res.json({
                                                        success: true,
                                                        msg: reason,
                                                        data: {
                                                            status: true,
                                                            paper_code: todaytimetable.paper_code,
                                                            secs,
                                                            gap: gap - secs,
                                                            docsLength
                                                        }
                                                    });
                                                } else {
                                                    return res.json({
                                                        success: true,
                                                        msg: reason,
                                                        data: {
                                                            status: false,
                                                            docsLength
                                                        }
                                                    });
                                                }
                                            }
                                            return res.json({ success: true });
                                        } else {
                                            return res.json({
                                                success: true,
                                                msg: 'Time Table Awaited!',
                                                data: {
                                                    status: false,
                                                    docsLength
                                                }
                                            });
                                        }
                                    }
                                }).maxTimeMS(20000);
                            } else {
                                return res.json({
                                    success: true,
                                    msg: 'No Subjects Alloted',
                                    data: {
                                        status: false,
                                        docsLength
                                    }
                                });
                            }
                        }
                    }).maxTimeMS(20000);
                }
            });
        } else {
            helper.errHandler(res, { message: 'Table Not Found' });
        }
    });

router.get('/student-subject-verification/:eno',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        // const eno = req.params.eno;
        const eno = req.user.enrollment_no;
        const Table = (getModel('student')[0]).TableModel;

        Table.findOne({ enrollment_no: eno }, { subjects: 1, branch: 1 }, (err, doc) => {
            if (err) {
                // console.log(err);
                res.status(500).json({ success: false, msg: "Network Error" });
            } else {
                if (!doc) {
                    res.status('404').json({
                        success: false,
                        msg: "Enrollment Not Found",
                        data: doc
                    })
                }
            }
        });
    });

module.exports = router;

// Handlers
function getModel(term) {
    const Model = Models.filter(o => o.TableName.toLowerCase() == term);
    return Model;
}

function shuffle(array) {
    var currentIndex = array.length, temporaryValue, randomIndex;

    // While there remain elements to shuffle...
    while (0 !== currentIndex) {

        // Pick a remaining element...
        randomIndex = Math.floor(Math.random() * currentIndex);
        currentIndex -= 1;

        // And swap it with the current element.
        temporaryValue = array[currentIndex];
        array[currentIndex] = array[randomIndex];
        array[randomIndex] = temporaryValue;
    }

    return array;
}
