// module.exports = _router;
var express = require("express");
var _router = express.Router();
var multer = require("multer");
var path = require("path");
var fs = require("fs");
var xlsx = require("node-xlsx");
// const IumsStudent = require('./../models/m_iums_student');
// const UserNotification = require('./../models/user-notification');
const Models = require('./../models/common');
const helper = require('./../Helpers/handler');
const rimraf = require('rimraf')
const passport = require('passport');

var store = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, "./uploads");
  },
  filename: function (req, file, cb) {
    cb(null, file.originalname + "_" + Date.now() + ".png");
  },
});

var upload = multer({ storage: store }).single("file");

var storeFileData = multer.diskStorage({
  destination: function (req, file, cb) {
    cb(null, "./data");
  },
  filename: function (req, file, cb) {
    cb(null, file.originalname + "_" + Date.now() + ".png");
  },
});

var uploadFileData = multer({ storage: storeFileData }).single("file");

var storeFaceData = multer.diskStorage({
  destination: function (req, file, cb) {
    if (!fs.existsSync('./faces/' + req.params.encId)) {
      fs.mkdirSync('./faces/' + req.params.encId);
    }
    cb(null, './faces/' + req.params.encId);
  },
  filename: function (req, file, cb) {
    cb(null, file.originalname + "_" + Date.now() + ".png");
  },
});

var uploadFaceData = multer({ storage: storeFaceData }).single("file");

_router.post("/upload", function (req, res, next) {
  upload(req, res, function (err) {
    if (err) {
      return res.status(501).json({ error: err });
    }
    //do all database record saving activity
    return res.json({ msg: "Uploaded Successfully", file: req.file });
  });
});

_router.get("/download/:filename", function (req, res, next) {
  filepath = path.join(__dirname, "/../uploads") + "/" + req.params.filename;
  res.sendFile(filepath);
});

_router.post("/uploadFace/:encId", function (req, res, next) {
  uploadFaceData(req, res, function (err) {
    if (err) {
      return res.status(501).json({ error: err });
    }
    //do all database record saving activity
    return res.json({ msg: "Uploaded Successfully", file: req.file });
  });
});

_router.get("/downloadFace/:filename", function (req, res, next) {
  filepath = path.join(__dirname, "/../uploads") + "/" + req.params.filename;
  res.sendFile(filepath);
});

_router.post("/uploadFileData", function (req, res, next) {
  uploadFileData(req, res, function (err) {
    if (err) {
      return res.status(501).json({ error: err });
    }
    //do all database record saving activity
    return res.json({ msg: "Uploaded Successfully", file: req.file });
  });
});

_router.get("/downloadFileData/:filename", function (req, res, next) {
  filepath = path.join(__dirname, "/../data") + "/" + req.params.filename;
  res.sendFile(filepath);
});

_router.get("/listFileData", function (req, res, next) {
  filepath = path.join(__dirname, "/../data") + "/";
  fs.readdir(filepath, (err, files) => {
    if (err) {
      return res.json({
        success: false,
        msg: err.message,
        data: err,
      });
    }
    return res.json({
      success: true,
      msg: "Listed Successfully",
      data: files,
    });
  });
});

_router.get("/deleteFileData/:filename", function (req, res, next) {
  filepath = path.join(__dirname, "/../data") + "/" + req.params.filename;
  fs.unlink(filepath, () => {
    res.json({
      success: true,
      msg: "Deleted Successfully",
      data: [],
    });
  });
});

_router.get(
  "/processFileData/:term/:filename/",
  passport.authenticate("jwt", { session: false }),
  (req, res, next) => {
    const SDfilename = req.params.filename;
    const term = req.params.term;
    const user = req.user;

    filepath = path.join(__dirname, "/../data") + "/" + req.params.filename;
    const sheets = xlsx.parse(filepath); // parses a file
    let fileData = [];
    let processedFileData = [];
    for (const sheet of sheets) {
      fileData = [...fileData, ...sheet.data];
    }
    // console.log(fileData);
    fileData.reverse();
    const headers = fileData.pop();
    fileData.reverse();
    const docs = [];
    for (let i = 0; i < fileData.length; i++) {
      const elem = fileData[i];
      let newDoc = {};
      for (let j = 0; j < headers.length; j++) {
        const key = headers[j];

        if (key == "options") {
          newDoc[key] = JSON.parse(elem[j]);
        } else {
          newDoc[key] = elem[j];
        }
      }
      docs.push(newDoc);
    }

    const Table = getModel(term)[0];

    if (!docs) {
      return helper.errHandler(res, { message: "Can't insert blank entry" });
    }
    if (!docs.length) {
      return helper.errHandler(res, { message: "Can't insert blank entry" });
    }

    if (Table) {
      if (term == "questionbank") {
        for (const doc of docs) {
          doc.created_at = Date.now();
          doc.created_by = req.user._id;
          if (!doc.institute_code) {
            doc.institute_code = req.user.institute_code;
          }

          let options = [];
          let options_hindi = [];

          for (let index = 1; index <= 4; index++) {
            options.push(doc['option' + index]);
            options_hindi.push(doc['option_hindi' + index]);
          }

          doc.options = options;
          doc.options_hindi = options_hindi;
        }
      } else {
        for (const doc of docs) {
          doc.created_at = Date.now();
          doc.created_by = req.user._id;
          if (!doc.institute_code) {
            doc.institute_code = req.user.institute_code;
          }
        }
      }

      Table.TableModel.insertMany(docs, (err, docs) => {
        if (err) {
          helper.errHandler(res, err);
        } else {
          res.json({
            success: true,
            msg: 'Data Inserted',
            data: {
              docsCount: docs.length
            }
          });
        }
      })
    } else {
      helper.errHandler(res, { message: 'Table Not Found' });
    }


    // res.json({
    //   success: true,
    //   msg: "Processind Data, You Will Be Notified When Done.",
    //   data: [],
    // });
  }
);

_router.get(
  "/processFileDataForMapping/:term/:filename/",
  passport.authenticate("jwt", { session: false }),
  (req, res, next) => {
    const SDfilename = req.params.filename;
    const term = req.params.term;
    const user = req.user;

    filepath = path.join(__dirname, "/../data") + "/" + req.params.filename;
    const sheets = xlsx.parse(filepath); // parses a file
    let fileData = [];
    let processedFileData = [];
    for (const sheet of sheets) {
      fileData = [...fileData, ...sheet.data];
    }
    // console.log(fileData);
    fileData.reverse();
    const headers = fileData.pop();
    fileData.reverse();
    const docs = [];
    for (let i = 0; i < fileData.length; i++) {
      const elem = fileData[i];
      let newDoc = {};
      for (let j = 0; j < headers.length; j++) {
        const key = headers[j];

        if (key == "options") {
          newDoc[key] = JSON.parse(elem[j]);
        } else {
          newDoc[key] = elem[j];
        }
      }
      docs.push(newDoc);
    }

    const Table = getModel(term)[0];

    if (!docs) {
      return helper.errHandler(res, { message: "Can't insert blank entry" });
    }
    if (!docs.length) {
      return helper.errHandler(res, { message: "Can't insert blank entry" });
    }

    if (Table) {
      if (term == "questionbank") {
        for (const doc of docs) {
          doc.created_at = Date.now();
          doc.created_by = req.user._id;
          if (!doc.institute_code) {
            doc.institute_code = req.user.institute_code;
          }

          let options = [];
          let options_hindi = [];

          for (let index = 1; index <= 4; index++) {
            options.push(doc['option' + index]);
            options_hindi.push(doc['option_hindi' + index]);
          }

          doc.options = options;
          doc.options_hindi = options_hindi;
        }
      } else {
        for (const doc of docs) {
          doc.created_at = Date.now();
          doc.created_by = req.user._id;
          if (!doc.institute_code) {
            doc.institute_code = req.user.institute_code;
          }
        }
      }

      Table.TableModel.insertMany(docs, (err, ques) => {
        if (err) {
          helper.errHandler(res, err);
        } else {
          let qIds = [];
          for (const doc of ques) {
            qIds.push(doc._id);
          }
          res.json({
            success: true,
            msg: 'Questions Processed',
            data: qIds

          });
        }
      })
    } else {
      helper.errHandler(res, { message: 'Table Not Found' });
    }


    // res.json({
    //   success: true,
    //   msg: "Processind Data, You Will Be Notified When Done.",
    //   data: [],
    // });
  }
);

_router.get(
  "/processPaperSetterFile/:filename/",
  passport.authenticate("jwt", { session: false }),
  (req, res, next) => {
    // const SDfilename = req.params.filename;
    // const term = req.params.term;
    // const user = req.user;

    filepath = path.join(__dirname, "/../data") + "/" + req.params.filename;
    const sheets = xlsx.parse(filepath); // parses a file

    sheets[0].term = 'testcreation';
    sheets[1].term = 'testpaper';
    sheets[2].term = 'questionbank';

    console.log(sheets);

    let insertCount = 0;

    // store inserted paper
    let insertedTestPaper = {};

    // to store question Ids
    let questionIds = [];

    // use to save question id's in paper
    let insertedQuestionCount = 0;
    let fetchedQuestionCount = 0;

    // to fetch existing question counts in database
    const qbTable = getModel('questionbank')[0];
    qbTable.TableModel.countDocuments((err, docs) => {
      if (err) {
        console.log('Unable to fetch Question Count');
      } else {
        if (docs) {
          fetchedQuestionCount = docs;
          console.log('Fetched Questions Count', fetchedQuestionCount);
        } else {
          console.log('Zero questions in Database');
        }
      }
    });


    // insert data into database when no duplicate name found in testpaper
    for (const sheet of sheets) {

      let fileData = [];
      let processedFileData = [];

      fileData = [...fileData, ...sheet.data]

      // console.log(fileData);
      fileData.reverse();
      const headers = fileData.pop();
      fileData.reverse();
      const docs = [];
      for (let i = 0; i < fileData.length; i++) {
        const elem = fileData[i];
        let newDoc = {};
        for (let j = 0; j < headers.length; j++) {
          const key = headers[j];

          if (key == "options") {
            newDoc[key] = JSON.parse(elem[j]);
          } else {
            newDoc[key] = elem[j];
          }
        }
        docs.push(newDoc);
      }

      const Table = getModel(sheet.term)[0];

      if (!docs) {
        return helper.errHandler(res, { message: "Can't insert blank entry" });
      }
      if (!docs.length) {
        return helper.errHandler(res, { message: "Can't insert blank entry" });
      }


      // check duplicate test paper name
      const tpTable = getModel('testpaper')[0];
      tpTable.TableModel.countDocuments({ test_paper_name: docs[0].test_paper_name }, (err, doc) => {
        if (err) {
          console.log('Error While Checking Duplicate Test Paper Name');
        } else {
          if (doc > 0) {
            res.json({
              success: true,
              msg: 'ERROR: ' + doc + ' Duplicate Paper Name Found! Please Change Paper Name in Excel File.',
              data: {
                count: doc
              }
            });
          } else {
            if (Table) {
              for (const doc of docs) {
                doc.created_at = Date.now();
                doc.created_by = req.user._id;
                if (!doc.institute_code) {
                  doc.institute_code = req.user.institute_code;
                }
              }
              Table.TableModel.insertMany(docs, (err, docs) => {
                if (err) {
                  helper.errHandler(res, err);
                } else {
                  /*  res.json({
                     success: true,
                     msg: 'Data Inserted',
                     data: {
                       docsCount: docs.length
                     }
                   }); */

                  insertCount++;
                  console.log(sheet.term + ' Data Inserted', docs.length);
                  // console.log('Insert Count', insertCount);

                  // store inserted test paper
                  if (sheet.term == 'testpaper') {
                    insertedTestPaper = docs[0];
                    console.log('Inserted Test Paper: ', insertedTestPaper);
                  }

                  // store length of inserted questions
                  if (sheet.term == 'questionbank') {
                    insertedQuestionCount = docs.length;
                    console.log('Inserted Questions: ', insertedQuestionCount);
                  }

                  // condition to check if all data sheets inserted
                  if (insertCount == 3) {

                    // to fetch inserted question Ids for mapping
                    Table.TableModel.find((err, docs) => {
                      if (err) {
                        console.log('Unable to fetch Questions');
                      } else {
                        if (docs) {
                          console.log('Questions: ', docs);
                          for (const doc of docs) {
                            questionIds.push(doc._id);
                          }
                          console.log('Question Ids: ', questionIds);

                          // to map question to test paper
                          const paperTable = getModel('testpaper')[0];
                          paperTable.TableModel.findOneAndUpdate({ test_paper_name: insertedTestPaper.test_paper_name }, { question_codes: questionIds }, (err, doc) => {
                            if (err) {
                              console.log('Unable to Map Questions');
                            } else {
                              if (doc) {
                                console.log('Questions Mapped Successfully');
                                res.json({
                                  success: true,
                                  msg: 'Data Inserted Successfully',
                                  data: {
                                    sheetCount: insertCount
                                  }
                                });
                              } else {
                                console.log('NO Data to Map Questions');
                              }
                            }
                          });
                        }
                      }
                    }).skip(fetchedQuestionCount);

                  }
                }
              })
            } else {
              helper.errHandler(res, { message: 'Table Not Found' });
            }
          }
        }
      });



    }






    // res.json({
    //   success: true,
    //   msg: "Processind Data, You Will Be Notified When Done.",
    //   data: [],
    // }); 
  }
);

_router.get(
  "/updateProcessFileData/:term/:filename/",
  // passport.authenticate("jwt", { session: false }),
  (req, res, next) => {
    const SDfilename = req.params.filename;
    const term = req.params.term;
    const user = req.user;

    filepath = path.join(__dirname, "/../data") + "/" + req.params.filename;
    const sheets = xlsx.parse(filepath); // parses a file
    let fileData = [];
    let processedFileData = [];
    for (const sheet of sheets) {
      fileData = [...fileData, ...sheet.data];
    }
    // console.log(fileData);
    fileData.reverse();
    const headers = fileData.pop();
    fileData.reverse();
    const docs = [];
    for (let i = 0; i < fileData.length; i++) {
      const elem = fileData[i];
      let newDoc = {};
      for (let j = 0; j < headers.length; j++) {
        const key = headers[j];

        if (key == "options") {
          newDoc[key] = JSON.parse(elem[j]);
        } else {
          newDoc[key] = elem[j];
        }
      }
      docs.push(newDoc);
    }

    const Table = getModel(term)[0];

    if (!docs) {
      return helper.errHandler(res, { message: "Can't insert blank entry" });
    }
    if (!docs.length) {
      return helper.errHandler(res, { message: "Can't insert blank entry" });
    }

    if (Table) {
      res.json({
        success: true,
        msg: 'Data Updating...',
        data: {
          docsCount: docs.length
        }
      });
      var i = 0;
      for (const doc of docs) {
        // console.log(doc);
        if (!doc.enrollment_no) {
          continue;
        }
        Table.TableModel.updateOne({ enrollment_no: doc.enrollment_no }, { $set: doc }, (err, docs) => {
          if (err) {
            console.log(err);
            console.log(err.message);
          } else {
            console.log(docs);
            console.log("updated");
            i++;
            console.log(i);
          }
        })
      }
    } else {
      helper.errHandler(res, { message: 'Table Not Found' });
    }


    // res.json({
    //   success: true,
    //   msg: "Processind Data, You Will Be Notified When Done.",
    //   data: [],
    // });
  }
);

_router.get("/listOldStudentData/:enrollmentNumber", function (req, res, next) {
  const enrollmentNumber = req.params.enrollmentNumber;
  filepath = path.join(__dirname, "/../uploads") + "/";
  fs.readdir(filepath, (err, files) => {
    if (err) {
      return res.json({
        success: false,
        msg: err.message,
        data: err,
      });
    }
    files = files.filter(o => o.search(enrollmentNumber) != '-1');
    return res.json({
      success: true,
      msg: "Listed Successfully",
      data: files,
    });
  });
});


// // AWS S3

const uploads3 = require('../services/file-upload');

const singleUploadQuestions3 = uploads3.uploadQuestiosns3.single('file');
const singleUploadStudentImages3 = uploads3.uploadStudentImages3.single('file');

_router.post('/uploadQuestion', (req, res) => {
  singleUploadQuestions3(req, res, (err) => {
    return res.json({ file: { 'filename': req.file ? req.file.location : '' } });
  });
});

_router.post('/uploadStudentImage',
  passport.authenticate('jwt', { session: false }),
  (req, res) => {
    singleUploadStudentImages3(req, res, (err) => {
      return res.json({ file: { 'filename': req.file ? req.file.location : '' } });
    });
  });

_router.get("/listStudentData/:enrollmentNumber", function (req, res, next) {
  const enrollmentNumber = req.params.enrollmentNumber;
  uploads3.s3.listObjects({ Bucket: 'rgpv-exam-images' }, function (err, data) {
    if (err) {
      //   console.log("Error", err);
      res.json(err);
    } else {
      //   console.log("Success", data);
      data = data.Contents;
      data = data.map(o => o.Key);
      data = data.filter(o => {
        o = o.split('/');
        if (o[0] && o[1]) {
          return o[1];
        } else {
          return false;
        }
      });
      data = data.map(o => {
        o = o.split('/');
        if (o[0] && o[1]) {
          return o[1];
        } else {
          return false;
        }
      });
      data = data.filter(o => {
        o = o.split('_');
        if (o[0] && o[1] && o[2] && (o[2] == enrollmentNumber + '.png' || o[2] == enrollmentNumber + '.jpg')) {
          return true;
        } else {
          return false;
        }
      });
      res.json({ success: true, data });
    }
  });
});

_router.get("/listAllStudentData/:enrollmentNumber", function (req, res, next) {
  const enrollmentNumber = req.params.enrollmentNumber;
  uploads3.s3.listObjectsV2({ Bucket: 'rgpv-exam-images', Prefix: 'studentimages/', MaxKeys: 1000 }, function (err, data) {
    if (err) {
      //   console.log("Error", err);
      res.json(err);
    } else {
      //   console.log("Success", data);
      data = data.Contents;
      data = data.map(o => o.Key);
      // data = data.filter(o => {
      //   o = o.split('/');
      //   if (o[0] && o[1]) {
      //     return o[1];
      //   } else {
      //     return false;
      //   }
      // });
      // data = data.map(o => {
      //   o = o.split('/');
      //   if (o[0] && o[1]) {
      //     return o[1];
      //   } else {
      //     return false;
      //   }
      // });
      // data = data.filter(o => {
      //   // o = o.split('_');
      //   // if (o[0] && o[1] && o[2] && (o[2] == enrollmentNumber + '.png' || o[2] == enrollmentNumber + '.jpg')) {
      //   if (o.search(enrollmentNumber) != -1) {
      //     return true;
      //   } else {
      //     return false;
      //   }
      // });
      res.json({ success: true, data });
    }
  });
});


module.exports = _router;

// Handlers
function getModel(term) {
  const Model = Models.filter(o => o.TableName.toLowerCase() == term);
  return Model;
}

// "0101CS161047","0101CS161048","0101CS161050","0101CS161051","0101CS161052","0101CS161053","0101CS161054","0101CS161056","0101CS161057","0101CS161058","0101CS161059","0101CS161061","0101CS161062","0101CS161063","0101CS161064","0101CS161065","0101CS161066","0101CS161067","0101CS161068","0101CS161069","0101CS161070","0101CS161072","0101CS161073","0101CS161074","0101CS161075","0101CS161076","0101CS161077","0101CS161078","0101CS161079","0101CS161080","0101CS161081","0101CS161082","0101CS161083","0101CS161084","0101CS161086","0101CS161087","0101CS161088","0101CS161089","0101CS161090","0101CS161091","0101CS161092","0101CS161093","0101CS161096","0101CS161097","0101CS161098","0101CS161099","0101CS161100","0101EC161101","0101EC161102","0101EC161104","0101EC161105","0101EC161106","0101EC161107","0101EC161108","0101EC161109","0101EC161110","0101EC161111","0101EC161112","0101EC161113","0101EC161114","0101EC161115","0101EC161116","0101EC161117","0101EC161119","0101EC161121","0101EC161123","0101EC161124","0101EC161125","0101EC163D15","0101EC163D19","0101EC163D22","0101EC173D01","0101EC173D02","0101EC173D03","0101EC173D04","0101EC173D05","0101EC173D06","0101EC173D07","0101EC173D08","0101EC173D09","0101EC173D11","0101EC173D12","0101EC173D13","0101EC173D14","0101EC173D15","0101EC173D16","0101EC173D17","0101EC173D18","0101EC173D19","0101EC173D21","0101EC173D22","0101EC173D23","0101EC173D24","0101EX121039","0101EX143D10","0101EX151043","0101ME161020","0101ME161021","0101ME161022","0101ME161024","0101ME161027","0101ME161029","0101ME161030","0101ME161032","0101ME161033","0101ME161034","0101ME161035","0101ME161036","0101ME161037","0101ME161038","0101ME161039","0101ME161041","0101ME161042","0101ME161044","0101ME161045","0101ME161047","0101ME161049","0101ME161051","0101ME161052","0101ME161053","0101ME161054","0101ME161056","0101ME161057","0101ME161058","0101ME161060","0101ME173D01","0101ME173D04","0101ME173D06","0101ME173D07","0101ME173D08","0101ME173D09","0101ME173D10","0101ME173D11","0101PC151005","0101PC161001","0101PC161002","0101PC161006","0101PC161008","0101PC161009","0101PC161010","0101PC161011","0101PC161014","0101PC161018","0101AU141032","0101AU151034","0101AU151052","0101AU151053","0101AU151056","0101AU161001","0101AU161002","0101AU161003","0101AU161004","0101AU161005","0101AU161008","0101AU161010","0101AU161013","0101AU161014","0101AU161017","0101AU161020","0101AU161021","0101AU161022","0101AU161023","0101AU161024","0101AU161025","0101AU161026","0101AU161028","0101AU161029","0101AU161032","0101AU161034","0101AU161035","0101AU161038","0101AU161040","0101AU161041","0101AU161044","0101AU161045","0101AU161046","0101AU161048","0101AU161049","0101AU161050","0101AU161051","0101AU161052","0101AU161056","0101AU161057","0101AU161058","0101AU161059","0101AU161060","0101AU161061","0101CE141020","0101CE141049","0101CE153D04","0101CE161001","0101CE161003","0101CS161101","0101CS161102","0101CS161103","0101CS161104","0101CS161105","0101CS161106","0101CS161107","0101CS161108","0101CS161110","0101CS161111","0101CS161113","0101CS161116","0101CS161117","0101CS161118","0101CS161119","0101CS161120","0101CS161121","0101CS161122","0101CS161123","0101CS161124","0101CS173D01","0101CS173D02","0101CS173D03","0101CS173D04","0101CS173D06","0101CS173D07","0101CS173D08","0101CS173D09","0101CS173D10","0101CS173D11","0101CS173D12","0101CS173D13","0101CS173D14","0101CS173D15","0101CS173D16","0101CS173D17","0101CS173D18","0101CS173D19","0101CS173D20","0101CS173D21","0101CS173D22","0101CS173D23","0101CS173D24","0101EC123D18","0101EC141094","0101EC141102","0101EC143D14","0101EC143D15","0101EC151011","0101EX151054","0101EX161001","0101EX161002","0101EX161003","0101EX161004","0101EX161006","0101EX161007","0101EX161010","0101EX161011","0101EX161012","0101EX161019","0101EX161020","0101EX161021","0101EX161022","0101EX161023","0101EX161024","0101EX161025","0101EX161026","0101EX161027","0101EX161028","0101EX161030","0101EX161031","0101EX161033","0101EX161034","0101EX161036","0101EX161037","0101EX161038","0101EX161039","0101EX161040","0101EX161041","0101EX161042","0101EX161044","0101EX161045","0101EX161046","0101EX161047","0101EX161048","0101EX161049","0101EX161050","0101EX161051","0101EX161052","0101EX161053","0101EX161054","0101EX161055","0101EX161056","0101EX161057","0101EX161058","0101PC161021","0101PC161023","0101PC161025","0101PC161027","0101PC161028","0101PC161030","0101PC161031","0101PC161032","0101PC161033","0101PC161034","0101PC161035","0101PC161036","0101PC161037","0101PC161038","0101PC161039","0101PC161042","0101PC161043","0101PC161044","0101PC161049","0101PC161051","0101PC161052","0101PC161053","0101PC173D03","0187EX161042","0308ME161040","0101CE161005","0101CE161006","0101CE161007","0101CE161008","0101CE161009","0101CE161012","0101CE161014","0101CE161015","0101CE161017","0101CE161018","0101CE161020","0101CE161021","0101CE161022","0101CE161023","0101CE161024","0101CE161026","0101CE161027","0101CE161028","0101CE161029","0101CE161030","0101CE161031","0101CE161032","0101CE161033","0101CE161035","0101CE161036","0101CE161038","0101CE161041","0101CE161042","0101CE161043","0101CE161044","0101CE161047","0101CE161048","0101CE161049","0101CE161050","0101CE161051","0101CE161052","0101CE161053","0101CE161054","0101CE161055","0101CE161056","0101CE161057","0101CE161059","0101CE161061","0101CE161062","0101CE161063","0101CE161064","0101CE161065","0101CE173D01","0101EC151019","0101EC151053","0101EC151069","0101EC151073","0101EC151074","0101EC151109","0101EC151124","0101EC153D05","0101EC153D23","0101EC161001","0101EC161003","0101EC161004","0101EC161006","0101EC161007","0101EC161008","0101EC161009","0101EC161011","0101EC161012","0101EC161013","0101EC161014","0101EC161015","0101EC161016","0101EC161017","0101EC161018","0101EC161019","0101EC161020","0101EC161021","0101EC161022","0101EC161023","0101EC161024","0101EC161025","0101EC161026","0101EC161027","0101EC161028","0101EC161029","0101EC161030","0101EC161031","0101EC161033","0101EC161034","0101EC161035","0101EC161036","0101EC161037","0101EC161038","0101EC161039","0101EC161040","0101EC161042","0101EC161044","0101EC161045","0101EX161059","0101EX161060","0101EX161061","0101EX161062","0101EX161063","0101EX173D02","0101EX173D03","0101EX173D04","0101EX173D05","0101EX173D08","0101EX173D09","0101EX173D12","0101IT151020","0101IT151048","0101IT161001","0101IT161002","0101IT161003","0101IT161005","0101IT161008","0101IT161009","0101IT161010","0101IT161011","0101IT161012","0101IT161015","0101IT161016","0101IT161017","0101IT161018","0101IT161019","0101IT161020","0101IT161021","0101IT161022","0101IT161023","0101IT161025","0101IT161026","0101IT161027","0101IT161028","0101IT161029","0101IT161030","0101IT161031","0101IT161032","0101IT161033","0101IT161034","0101IT161035","0101IT161036","0101IT161037","0101IT161038","0101IT161039","0101IT161040","0101CE173D02","0101CE173D03","0101CE173D04","0101CE173D05","0101CE173D06","0101CE173D07","0101CE173D09","0101CE173D10","0101CE173D11","0101CE173D12","0101CS141102","0101CS161001","0101CS161002","0101CS161003","0101CS161004","0101CS161006","0101CS161007","0101CS161008","0101CS161009","0101CS161010","0101CS161011","0101CS161012","0101CS161013","0101CS161014","0101CS161015","0101CS161016","0101CS161017","0101CS161019","0101CS161020","0101CS161021","0101CS161022","0101CS161023","0101CS161024","0101CS161025","0101CS161027","0101CS161029","0101CS161031","0101CS161032","0101CS161036","0101CS161037","0101CS161038","0101CS161039","0101CS161040","0101CS161041","0101CS161042","0101CS161043","0101CS161045","0101EC161046","0101EC161048","0101EC161049","0101EC161050","0101EC161051","0101EC161052","0101EC161053","0101EC161054","0101EC161055","0101EC161057","0101EC161058","0101EC161059","0101EC161060","0101EC161061","0101EC161062","0101EC161063","0101EC161064","0101EC161065","0101EC161068","0101EC161069","0101EC161070","0101EC161071","0101EC161072","0101EC161074","0101EC161075","0101EC161076","0101EC161077","0101EC161078","0101EC161079","0101EC161080","0101EC161083","0101EC161084","0101EC161086","0101EC161087","0101EC161088","0101EC161090","0101EC161091","0101EC161092","0101EC161093","0101EC161094","0101EC161095","0101EC161096","0101EC161097","0101EC161098","0101EC161099","0101EC161100","0101IT161041","0101IT161042","0101IT161044","0101IT161045","0101IT161046","0101IT161047","0101IT161048","0101IT161049","0101IT161050","0101IT161051","0101IT161052","0101IT161053","0101IT161054","0101IT161055","0101IT161056","0101IT161057","0101IT161058","0101IT161059","0101IT161060","0101IT161062","0101IT163D02","0101IT163D09","0101IT173D01","0101IT173D02","0101IT173D03","0101IT173D05","0101IT173D09","0101IT173D10","0101IT173D11","0101IT173D12","0101ME161001","0101ME161002","0101ME161003","0101ME161004","0101ME161005","0101ME161006","0101ME161007","0101ME161008","0101ME161009","0101ME161010","0101ME161011","0101ME161012","0101ME161014","0101ME161015","0101ME161016","0101ME161017","0101ME161018","0101ME161019"
