const _ = require('lodash');
const express = require('express');
const router = express.Router();
const bcrypt = require('bcryptjs');
const jwt = require('jsonwebtoken');
const passport = require('passport');
const User = require('../models/user');
const config = require('../config/database');

router.post('/getUserById', (req, res) => {
    const userid = req.body.id;
    User.getUserById(userid, (err, user) => {
        if (err) {
            res.json({ success: true, msg: JSON.stringify(err) });
        } else {
            if (user) {
                res.json({ success: true, msg: 'User Fetched', user: user });
            } else {
                res.json({ success: false, msg: 'User not found' });
            }
        }
    });
});

router.get('/getAllUsersCount', (req, res) => {
    User.getAllUsersCount((err, count) => {
        if (err) {
            return res.json({ success: false, msg: 'Error in get users', err: err });
        } else {
            if (count) {
                return res.json({ success: true, msg: 'Count get successfully', count: count })
            } else {
                return res.json({ success: false, msg: 'Something went wrong' })
            }
        }
    });
});

router.get('/getAllUsers',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        User.getUsers((err, users) => {
            if (err) {
                res.json({ success: false, msg: 'Tnternal Server Error', err: err });
            } else {
                if (users.length) {
                    res.json({ success: true, msg: 'Users Fetched', users: users });
                } else {
                    res.json({ success: true, msg: "No Users Found", users: users });
                }
            }
        });
    });

router.get('/getAllUsersSkipLimit/:page/:limit', (req, res) => {
    const page = parseInt(req.params.page);
    const limit = parseInt(req.params.limit);
    const skip = parseInt(page * limit - limit);
    User.getUsersSkipLimit(skip, limit, (err, users) => {
        if (err) {
            res.json({ success: false, msg: 'Tnternal Server Error', err: err });
        } else {
            if (users.length) {
                res.json({ success: true, msg: 'Users Fetched', users: users });
            } else {
                res.json({ success: true, msg: "No Users Found", users: users });
            }
        }
    });
})

router.post('/getUsersByRole', (req, res) => {
    const role = req.body.role;
    User.getUsersByRole(role, (err, users) => {
        if (err) {
            res.json({ success: false, msg: 'Internal Server Error', err: err });
        } else {
            if (users.length) {
                res.json({ success: true, msg: 'Users Fetched', users: users });
            } else {
                res.json({ success: true, msg: "No Users Found", users: users });
            }
        }
    });
})

router.post('/getUsersByOwner', (req, res) => {
    const owner = req.body.owner;
    User.getUsersByOwner(owner, (err, users) => {
        if (err) {
            res.json({ success: false, msg: 'Tnternal Server Error', err: err });
        } else {
            if (users.length) {
                res.json({ success: true, msg: 'Users Fetched', users: users });
            } else {
                res.json({ success: true, msg: "No Users Found", users: users });
            }
        }
    });
})

router.post('/getUsersByRoleAndOwner', (req, res) => {
    const role = req.body.role;
    const owner = req.body.owner;
    User.getUsersByRoleAndOwner(role, owner, (err, users) => {
        if (err) {
            res.json({ success: false, msg: 'Tnternal Server Error', err: err });
        } else {
            if (users.length) {
                res.json({ success: true, msg: 'Users Fetched', users: users });
            } else {
                res.json({ success: true, msg: "No Users Found", users: users });
            }
        }
    });
})

// // get Office percentage by admin
// router.get('/get-officePercentage/:admin', passport.authenticate('jwt', { session: false }), (req, res) => {
//   // // console.log(req.params.admin);
//   User.getUserByUsername(req.params.admin, (err, user) => {
//     if (err) {
//       res.status(200).json({ success: false, msg: 'error occured', err: err });
//     } else {
//       if (user) {
//         if (user.officeId) {
//           Office.getOfficeById(user.officeId, (err, office) => {
//             if (office) {
//               res.status(200).json({
//                 success: true,
//                 msg: 'office percentage found',
//                 per: office.officePercentage,
//                 officeId: office._id
//               });
//             } else {
//               res
//                 .status(200)
//                 .json({ success: false, msg: 'office does not found' });
//             }
//           });
//         } else {
//           res
//             .status(200)
//             .json({ success: false, msg: 'User does not contain office id' });
//         }
//       } else {
//         res.status(200).json({ success: false, msg: 'User not found' });
//       }
//     }
//   });
// });

// get counsellors
router.get(
    '/getCounsellors', passport.authenticate('jwt', { session: false }),
    (req, res) => {
        let counsellors = [];
        User.getCounsellors((err, counsellors) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                res.status(200).json({
                    success: true,
                    msg: 'Counsellors found',
                    counsellors: counsellors
                });
            }
        });
    }
);

// get counsellors by username
router.get(
    '/getCounsellors/:username',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        let counsellors = [];
        let username = req.params.username;
        User.getCounsellorsOf(username, (err, counsellors) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                res.status(200).json({
                    success: true,
                    msg: 'Counsellors found',
                    counsellors: counsellors
                });
            }
        });
    }
);

// get counsellors of manager
router.get(
    '/getCounsellorsOfManager/:username',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        let counsellors = [];
        let username = req.params.username;
        // console.log(username);
        User.getCounsellorsOfManager(username, (err, counsellors) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                // console.log("Counsellors: ", counsellors);
                res.status(200).json({
                    success: true,
                    msg: 'Counsellors found',
                    counsellors: counsellors
                });
            }
        });
    }
);




// get managers
router.get(
    '/getManagers',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        let managers = [];
        User.getManagers((err, managers) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                res
                    .status(200)
                    .json({ success: true, msg: 'managers found', managers: managers });
            }
        });
    }
);

// get manager by username
router.get(
    '/getManagers/:username',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        let managers = [];
        let username = req.params.username;
        User.getManagersOfAdmin(username, (err, managers) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                res
                    .status(200)
                    .json({ success: true, msg: 'managers found', managers: managers });
            }
        });
    }
);

// get admins
router.get(
    '/getAdmins',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        User.getAdmins((err, admins) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                res.status(200).json({
                    success: true,
                    msg: 'Admins found',
                    admins: admins
                });
            }
        });
    }
);

router.get('/get-user/:username', passport.authenticate('jwt', { session: false }), (req, res) => {
    User.getUserByUsername(req.params.username, (err, user) => {
        if (err) {
            res.status(200).json({ success: false, msg: 'error occured', err: err });
        } else {
            if (user) {
                res.status(200).json({ success: true, msg: 'User found', user: user });
            } else {
                res.status(200).json({ success: false, msg: 'User Not found' });
            }
        }
    });
});

router.get('/getofficeId/:username', passport.authenticate('jwt', { session: false }), (req, res) => {
    User.getUserByUsername(req.params.username, (err, user) => {
        if (err) {
            res.status(200).json({ success: false, msg: 'error occured', err: err });
        } else {
            if (user) {
                res
                    .status(200)
                    .json({ success: true, msg: 'Office Id found', id: user.officeId });
            } else {
                res.status(200).json({ success: false, msg: 'officeId Not found' });
            }
        }
    });
});

//   username: { type: String, required: true },
//   name: { type: String, required: true },
//   gender: { type: String, required: true },
//   wallet: { type: JSON },
//   officeId: { type: String },
//   cities: { type: Array, default: [] },
//   role: { type: String, required: true },
//   leads: { type: Array, default: [] },
//   owner: { type: JSON, required: true },
//   password: { type: String, required: true },
//   hashPassword:

// edit user
router.post(
    '/edit-user',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        User.getUserByUsername(req.body.username, (err, user) => {
            if (err) {
                res.json({ success: false, msg: 'Internal Server Error', err: err });
            } else {
                if (user) {
                    if (user._id.toString() !== req.body._id) {
                        res.json({ success: false, msg: 'Username already exists', err: err });
                    } else {
                        User.getUserById(req.body._id, (err, user) => {
                            if (err) {
                                res
                                    .status(200)
                                    .json({ success: false, msg: 'error occured', err: err });
                            } else {
                                if (user) {
                                    if (user) {
                                        // console.log(profile);
                                        user.name = req.body.name;
                                        user.username = req.body.username;
                                        user.gender = req.body.gender;
                                        user.mobile = req.body.mobile;
                                        user.email = req.body.email;
                                        user.role = req.body.role;
                                        user.cities = req.body.cities;
                                        user.offices = req.body.offices;
                                        user.owner = req.body.owner;
                                        user.callerId = req.body.callerId;
                                        user.institute_code = req.body.institute_code;


                                        bcrypt.hash(req.body.password, 10, (err, hash) => {
                                            if (err) {
                                                return res.status(200).json({
                                                    success: false,
                                                    msg: 'Error hashing password'
                                                });
                                            }

                                            user.password = req.body.password;
                                            user.hashPassword = hash;

                                            User.addUser(user, (err, user) => {
                                                if (err) {
                                                    res
                                                        .status(200)
                                                        .json({ success: false, msg: 'error occured', err: err });
                                                } else {

                                                    //   const newActivity = new Activity({
                                                    //     activityType: 'User Edited',
                                                    //     activityData: {
                                                    //       oldProfile: newObj,
                                                    //       newProfile: user
                                                    //     },
                                                    //     User: req.user,
                                                    //     UserId: req.user._id,
                                                    //     LeadId: user._id,
                                                    //     msg: `${user.username} Details Edited By ${req.user.username}`,
                                                    //   });
                                                    //   Activity.addActivity(newActivity, (err, data) => {
                                                    //     if (err) throw err;
                                                    //   });

                                                    res.status(200).json({
                                                        success: true,
                                                        msg: 'User edit successful.',
                                                        user: user
                                                    });
                                                }
                                            });
                                        });
                                    } else {
                                        res
                                            .status(200)
                                            .json({ success: false, msg: 'error in updating profile' });
                                    }
                                } else {
                                    res.status(200).json({ success: false, msg: 'user not found' });
                                }
                            }
                        });
                    }
                } else {
                    User.getUserById(req.body._id, (err, user) => {
                        if (err) {
                            res
                                .status(200)
                                .json({ success: false, msg: 'error occured', err: err });
                        } else {
                            if (user) {
                                // console.log(profile);
                                user.name = req.body.name;
                                user.username = req.body.username;
                                user.gender = req.body.gender;
                                user.email = req.body.email;
                                user.mobile = req.body.mobile;
                                user.role = req.body.role;
                                user.cities = req.body.cities;
                                user.offices = req.body.offices;
                                user.owner = req.body.owner;
                                user.callerId = req.body.callerId;

                                User.addUser(user, (err, user) => {
                                    if (err) {
                                        res
                                            .status(200)
                                            .json({ success: false, msg: 'error occured', err: err });
                                    } else {

                                        //   const newActivity = new Activity({
                                        //     activityType: 'User Edited',
                                        //     activityData: {
                                        //       oldProfile: newObj,
                                        //       newProfile: user
                                        //     },
                                        //     User: req.user,
                                        //     UserId: req.user._id,
                                        //     LeadId: user._id,
                                        //     msg: `${user.username} Details Edited By ${req.user.username}`,
                                        //   });
                                        //   Activity.addActivity(newActivity, (err, data) => {
                                        //     if (err) throw err;
                                        //   });

                                        res.status(200).json({
                                            success: true,
                                            msg: 'User edit successful.',
                                            user: user
                                        });
                                    }
                                });
                            } else {
                                res.status(200).json({ success: false, msg: 'user not found' });
                            }
                        }
                    });
                }
            }
        });
    }
);

// update Password
router.post(
    '/change-password',
    // passport.authenticate('jwt', { session: false }),
    (req, res) => {
        // console.log(req.body.username);
        // console.log(req.body.newPassword);
        // console.log(req.body.currentPassword);
        User.getUserByUsername(req.body.username, (err, user) => {
            if (err) {
                res.status(200).json({
                    success: false,
                    msg: 'Error in changing password',
                    err: err
                });
            } else {
                if (user) {
                    User.comparePassword(
                        req.body.currentPassword,
                        user.hashPassword,
                        (err, isMatch) => {
                            if (err) {
                                res.status(200).json({
                                    success: false,
                                    msg: 'Error in current checking password',
                                    err: err
                                });
                            } else {
                                if (isMatch) {
                                    bcrypt.hash(req.body.newPassword, 10, (err, hash) => {
                                        if (err) {
                                            return res.status(200).json({
                                                success: false,
                                                msg: 'Error hashing password'
                                            });
                                        }

                                        user.password = req.body.newPassword;
                                        user.hashPassword = hash;

                                        User.addUser(user, (err, user) => {
                                            if (err) {
                                                res.status(500).json({
                                                    success: false,
                                                    msg: 'Failed to update password',
                                                    err: JSON.stringify(err)
                                                });
                                            } else {

                                                // const newActivity = new Activity({
                                                //   activityType: 'Password Changed',
                                                //   activityData: {
                                                //     oldPassword: req.body.currentPassword
                                                //   },
                                                //   User: req.user,
                                                //   UserId: req.user._id,
                                                //   LeadId: user._id,
                                                //   msg: `${user.username} Updated password`,
                                                // });
                                                // Activity.addActivity(newActivity, (err, data) => {
                                                //   if (err) throw err;
                                                // });

                                                res
                                                    .status(201)
                                                    .json({ success: true, msg: 'password updated' });
                                            }
                                        });
                                    });
                                } else {
                                    res.status(200).json({
                                        success: false,
                                        msg: 'Your current password is incorrect'
                                    });
                                }
                            }
                        }
                    );
                } else {
                    res.status(200).json({ success: false, msg: 'user not found' });
                }
            }
        });
    }
);

// change password
router.post('/change-password', (req, res) => {
    User.getUserByUsername(req.body.username, (err, user) => {
        if (err) {
            res.json({ success: false, msg: JSON.stringify(err) });
        } else {
            if (user) {
                bcrypt.hash(req.body.password, 10, (err, hash) => {
                    if (err) {
                        return res
                            .status(200)
                            .json({ success: false, msg: 'Error hashing password' });
                    }
                    User.updateOne({ username: req.body.username }, { $set: { password: req.body.password, hashPassword: hash } }, (err, updated) => {
                        if (err) {
                            return res
                                .status(200)
                                .json({ success: false, msg: 'Something went wrong' });
                        } else {
                            return res.json({ success: true, msg: 'Password updated successfully.' })
                        }
                    });
                });
            } else {
                return res.json({ success: false, msg: `user not found.` });
            }
        }
    });
});

// create user
router.post(
    '/createUser',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        User.getUserByUsername(req.body.username, (err, user) => {
            if (err) {
                res.json({ success: false, msg: JSON.stringify(err) });
            } else {
                if (user) {
                    return res.json({ success: false, msg: `username already exist.` });
                } else {
                    bcrypt.hash(req.body.password, 10, (err, hash) => {
                        if (err) {
                            return res
                                .status(200)
                                .json({ success: false, msg: 'Error hashing password' });
                        }
                        let newUser = new User({
                            name: req.body.name,
                            username: req.body.username,
                            password: req.body.password,
                            gender: req.body.gender,
                            email: req.body.email,
                            mobile: req.body.mobile,
                            callerId: req.body.callerId,
                            role: req.body.role,
                            cities: req.body.city,
                            owner: req.body.owner,
                            hashPassword: hash,
                            officeId: req.body.officeId,
                            role: req.body.role,
                            institute_code: req.body.institute_code,
                            created_by: req.user._id,
                        });

                        if (!req.body.institute_code) {
                            newUser.institute_code = req.body.institute_code
                        }

                        User.addUser(newUser, (err, user) => {
                            if (err) {
                                res.status(500).json({
                                    success: false,
                                    msg: 'Failed to add user',
                                    err: JSON.stringify(err)
                                });
                            } else {

                                // const newActivity = new Activity({
                                //   activityType: 'User Created',
                                //   activityData: {
                                //     newUser: newUser
                                //   },
                                //   User: req.user,
                                //   UserId: req.user._id,
                                //   LeadId: user._id,
                                //   msg: `${newUser.username} User Created By ${req.user.username}`,
                                // });
                                // Activity.addActivity(newActivity, (err, data) => {
                                //   if (err) throw err;
                                // });

                                res
                                    .status(201)
                                    .json({ success: true, user: user, msg: 'user Added' });
                            }
                        });
                    });
                }
            }
        });
    }
);

// login
router.post('/login', (req, res) => {
    const username = req.body.username;
    const password = req.body.password;
    // console.log(username, password);
    User.getUserByUsername(req.body.username, (err, user) => {
        if (err) {
            res.status(200).json({ success: false, msg: 'error occured', err: err });
        } else {
            if (user) {
                User.comparePassword(password, user.hashPassword, (err, isMatch) => {
                    if (err) {
                        // // console.log(err);
                        res
                            .status(200)
                            .json({ success: false, msg: 'error occured', err: err });
                    }
                    if (isMatch) {
                        // console.log(user);
                        var t_user = {
                            id: user._id,
                            name: user.name,
                            username: user.username,
                            owner: user.owner,
                            gender: user.gender,
                            role: user.role,
                            callerId: user.callerId,
                            institute_code: user.institute_code,
                        };
                        const token = jwt.sign(t_user, config.secret, {
                            expiresIn: 604800
                        });
                        res.status(200).json({
                            success: true,
                            msg: 'Welcome ' + user.name,
                            token: 'JWT ' + token,
                            user: {
                                id: user._id,
                                name: user.name,
                                username: user.username,
                                owner: user.owner,
                                gender: user.gender,
                                role: user.role,
                                officeId: user.officeId,
                                wallet: user.wallet,
                                callerId: user.callerId,
                                institute_code: user.institute_code,
                            }
                        });
                    } else {
                        res.status(200).json({ success: false, msg: 'wrong password' });
                    }
                });
            } else {
                res.status(200).json({ success: false, msg: 'User not found' });
            }
        }
    });
});

// login as admin
router.post('/loginAsAdmin',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        const username = req.body.username;
        const adminUname = req.body.adminUname;
        // console.log(username, password);
        User.getUserByUsername(username, (err, user) => {
            if (err) {
                res.status(200).json({ success: false, msg: 'error occured', err: err });
            } else {
                if (user) {
                    User.getUserByUsername(adminUname, (err, aUser) => {
                        if (err) {
                            res.status(200).json({ success: false, msg: 'error occured', err: err });
                        } else {
                            if (user) {
                                // console.log(user);
                                var t_user = {
                                    id: user._id,
                                    name: user.name,
                                    username: user.username,
                                    owner: user.owner,
                                    gender: user.gender,
                                    role: user.role,
                                    callerId: user.callerId,
                                    institute_code: user.institute_code,
                                };
                                const token = jwt.sign(t_user, config.secret, {
                                    expiresIn: 604800
                                });

                                // const newActivity = new Activity({
                                //   activityType: 'Login As Admin',
                                //   activityData: {
                                //     from: adminUname,
                                //     to: username
                                //   },
                                //   User: req.user,
                                //   UserId: req.user._id,
                                //   LeadId: user._id,
                                //   msg: `${adminUname} Logged In as admin to ${username}`,
                                // });
                                // Activity.addActivity(newActivity, (err, data) => {
                                //   if (err) throw err;
                                // });

                                res.status(200).json({
                                    success: true,
                                    msg: 'Welcome ' + user.name,
                                    token: 'JWT ' + token,
                                    user: {
                                        id: user._id,
                                        name: user.name,
                                        username: user.username,
                                        owner: user.owner,
                                        gender: user.gender,
                                        role: user.role,
                                        officeId: user.officeId,
                                        wallet: user.wallet,
                                        callerId: user.callerId,
                                        institute_code: user.institute_code,
                                    }
                                });
                            } else {
                                res.status(200).json({ success: false, msg: 'Admin not found' });
                            }
                        }
                    });
                } else {
                    res.status(200).json({ success: false, msg: 'User not found' });
                }
            }
        });
    });

// delete user
router.post(
    '/deleteUser',
    passport.authenticate('jwt', { session: false }),
    (req, res) => {
        User.getUserByUsername(req.body.username, (err, user) => {
            if (err) {
                res
                    .status(200)
                    .json({ success: false, msg: 'error occured', err: err });
            } else {
                if (user) {
                    User.deleteUser(req.body.username, (err, user) => {
                        if (err) {
                            res
                                .status(200)
                                .json({ success: false, msg: 'error occured', err: err });
                        } else {

                            //   const newActivity = new Activity({
                            //     activityType: 'User Deleted',
                            //     activityData: {
                            //       user: user
                            //     },
                            //     User: req.user,
                            //     UserId: req.user._id,
                            //     LeadId: user._id,
                            //     msg: `${req.body.username} Deleted By ${req.user.username}`,
                            //   });
                            //   Activity.addActivity(newActivity, (err, data) => {
                            //     if (err) throw err;
                            //   });

                            res.status(200).json({
                                success: true,
                                msg: 'user deleted successfully..',
                                user: user
                            });
                        }
                    });
                } else {
                    res.status(200).json({ success: false, msg: 'User not found' });
                }
            }
        });
    }
);

// Function to generate OTP 
// function generateOTP() {

//   // Declare a digits variable  
//   // which stores all digits 
//   var digits = '0123456789ABCDEFGHIJ';
//   let OTP = '';
//   for (let i = 0; i < 6; i++) {
//       OTP += digits[Math.floor(Math.random() * 10 + Math.random() * 10)];
//   }
//   return OTP;
// }

// router.post('/sendOTP', (req, res) => {
//   console.log('req recieved');

//   // const otp = req.body.otp;
//   const website = req.body.website;
//   const mobile = req.body.mobile;
//   const otp = req.body.otp;

//   const msg = `Hi, Welcome to ${website}.com. Your mobile verification code is ${otp}. www.${website}.com`;

//   var url = `http://125.16.147.178:80/VoicenSMS/webresources/CreateSMSCampaignGet?ukey=jUZQZMxsU54lG9XiQQQmbLQkLPNUT7VRXBWE7mAHa4=&msisdn={ NUMBER }&language=0&credittype=2&senderid=SBWALE&templateid=0&message={ MESSAGE }`

//   var arr = url.split("{ NUMBER }");
//   var arr2 = arr[1].split("{ MESSAGE }")[0] + `${msg}`;
//   arr = arr[0] + `${mobile}`;
//   const newUrl = arr + arr2;
//   request(newUrl, function (error, response, body) {
//     // console.log(response);
//     console.log(error);
//     if (error) {
//       return res.json({ success: false, msg: 'some error occured, try again later', data: error });
//     } else {
//     //   const newActivity = new Activity({
//     //     activityType: `OTP Sent Via SMS on ${mobile}`,
//     //     activityData: {
//     //       mobile: mobile,
//     //       msg: msg,
//     //       url: newUrl,
//     //     },
//     //     website: website,
//     //     otp: otp,
//     //     msg: `OTP Sent To ${mobile} From ${website}`,
//     //   });
//     //   Activity.addActivity(newActivity, (err, data) => {
//     //     if (err) throw err;
//     //   });
//       return res.json({ success: true, msg: 'OTP sent successfully', data: response });
//     }
//   });
// });

module.exports = router;
