"use strict"
const nodemailer = require("nodemailer");
const Handler = require("./../Helpers/handler");
const Config = require("./../config");

const Transporter = nodemailer.createTransport({
    host: Config.mail.host,
    port: Config.mail.port,
    secure: false,
    auth: {
        user: Config.mail.user,
        pass: Config.mail.pass
    }
});

const Transporters = [];
for (const m of Config.mails) {
    const t = nodemailer.createTransport({
        host: m.host,
        port: m.port,
        secure: false,
        auth: {
            user: m.user,
            pass: m.pass
        }
    });
    Transporters.push(t);
}

if (!Transporters.length) {
    console.log('no mails configuration found!!!');
}

var position = 0;

module.exports.sendMail = async ({ email, subject, body }, callback) => {
    const mailInfo = Handler.getModel('mailinfo');
    const errorLog = Handler.getModel('errorlog');
    try {
        const info = await Transporters[position].sendMail({
            from: Config.mails[position].user,
            to: Config.production ? email : Config.testmail,
            subject,
            html: body
        });
        mailInfo.TableModel.create({ messageId: info.messageId, info, created_at: Date.now() }, (err, doc) => {
            callback(err, doc);
        });
    } catch (error) {
        errorLog.TableModel.create({ message: "Mail Error", error, created_at: Date.now() });
        callback(error, null);
    }
    console.log(position);
    position++;
    if (position == Transporters.length) {
        position = 0;
    }
}